/*
 *
 *
 *  $('#id').jsonSuggest({data: JSONDATA, minCharacters: 3, onSelect: function(){} });
 */

(function($) {

    $.fn.jsonSuggest = function(settings) {
        var defaults = {
                url: '',
                data: [],
                minCharacters: 1,
                maxResults: undefined,
                wildCard: '',
                caseSensitive: false,
                notCharacter: '!',
                headMsg: '',
                maxHeight: 350,
                onSelect: undefined
            }, getJSONTimeout;
        settings = $.extend(defaults, settings);

        return this.each(function() {
            /**
             * Escape some text so that it can be used inside a regular expression
             * without implying regular expression rules iself.
             */
            function regexEscape(txt, omit) {
                var specials = ['/', '.', '*', '+', '?', '|',
                    '(', ')', '[', ']', '{', '}', '\\'];

                if (omit) {
                    for (var i = 0; i < specials.length; i++) {
                        if (specials[i] === omit) { specials.splice(i,1); }
                    }
                }

                var escapePatt = new RegExp('(\\' + specials.join('|\\') + ')', 'g');
                return txt.replace(escapePatt, '\\$1');
            }

            var obj = $(this),
                wildCardPatt = new RegExp(regexEscape(settings.wildCard || ''),'g'),
                results = $('<ul />'),
                currentSelection;

            /**
             * When an item has been selected then update the input box,
             * hide the results again and if set, call the onSelect function.
             */
            function selectResultItem(item) {
                obj.val(item.name);
                $(results).html('').hide();

                if (typeof settings.onSelect === 'function') {
                    settings.onSelect(item);
                }
            }

            /**
             * Build the results HTML based on an array of objects that matched
             * the search criteria
             */
            function buildResults(resultObjects, filterTxt) {
                filterTxt = '(' + filterTxt + ')';

                var bOddRow = true, i, iFound = 0,
                    filterPatt = settings.caseSensitive ? new RegExp(filterTxt, 'g') : new RegExp(filterTxt, 'ig');

                $(results).html(settings.headMsg).hide();

                for (i = 0; i < resultObjects.length; i += 1) {
                    var item = $('<li />');

                    $(item).append( resultObjects[i].name );

                    $(item).
                        addClass((bOddRow) ? 'odd' : 'even').
                        click((function(n) { return function() {
                            selectResultItem(resultObjects[n]);
                        };})(i));

                    $(results).append(item);

                    bOddRow = !bOddRow;

                    iFound += 1;
                    if (typeof settings.maxResults === 'number' && iFound >= settings.maxResults) {
                        break;
                    }
                }

                if ($('li', results).length > 0) {
                    currentSelection = undefined;
                    $(results).show().css('height', 'auto');

                    if ($(results).height() > settings.maxHeight) {
                        $(results).css({'overflow': 'hidden', 'height': settings.maxHeight + 'px'});
                        $(this).find("ul").scroller({
                            scrollBars: true,
                            verticalScroll: true,
                            horizontalScroll: false,
                            vScrollCSS: "jqmScrollbar",
                            useJsScroll: !$.feat.nativeTouchScroll,
                            noParent: $.feat.nativeTouchScroll
                        });
                    }
                } else {
                    $(results).html("<p>Nessun dato trovato con questo nome!</p>").show();
                }
            }

            /**
             * Prepare the search data based on the settings for this plugin,
             * run a match against each item in the possible results and display any
             * results on the page allowing selection by the user.
             */
            function runSuggest(e) {
                var search = function(searchData) {
                    if (this.value.length < settings.minCharacters) {
                        $(results).html('').hide();
                        return false;
                    }

                    var resultObjects = [],
                        filterTxt = (!settings.wildCard) ? regexEscape(this.value) : regexEscape(this.value, settings.wildCard).replace(wildCardPatt, '.*'),
                        bMatch = true,
                        filterPatt, i;

                    if (settings.notCharacter && filterTxt.indexOf(settings.notCharacter) === 0) {
                        filterTxt = filterTxt.substr(settings.notCharacter.length,filterTxt.length);
                        if (filterTxt.length > 0) { bMatch = false; }
                    }
                    filterTxt = filterTxt || '.*';
                    filterTxt = settings.wildCard ? '^' + filterTxt : filterTxt;
                    filterPatt = settings.caseSensitive ? new RegExp(filterTxt) : new RegExp(filterTxt, 'i');


                    // Look for the required match against each single search data item. When the not
                    // character is used we are looking for a false match.
                    for (i in searchData) {
                        var searchDataString = searchData[i].name;
                        if (filterPatt.test(searchDataString) === bMatch) {
                            resultObjects.push(searchData[i]);
                        }
                    }

                    buildResults(resultObjects, filterTxt);
                };

                if (settings.data) {
                    search.apply(this, [settings.data]);
                } else if (settings.url && typeof settings.url === 'string') {
                    if (this.value.length < settings.minCharacters) {
                        $(results).html('').hide();
                        return false;
                    }
                    var text = this.value;
                    $(results).html('<li>Ricerca in corso...</li>').
                        show().css('height', 'auto');

                    getJSONTimeout = window.clearTimeout(getJSONTimeout);
                    getJSONTimeout = window.setTimeout(function() {
                        $.getJSON(settings.url, {search: text}, function(data) {
                            if (data) {
                                buildResults(data.tags, text);
                            } else {
                                $(results).html('').hide();
                            }
                        });
                    }, 500);
                }
            }

            /**
             * To call specific actions based on the keys pressed in the input
             * box. Special keys are up, down and return. All other keys
             * act as normal.
             */
            function keyListener(e) {
                switch (e.keyCode) {
                    case 13: // return key
                        $(currentSelection).trigger('click');
                        return false;
                    default:
                        runSuggest.apply(this, [e]);
                }
            }

            // Prepare the input box to show suggest results by adding in the events
            // that will initiate the search and placing the element on the page
            // that will show the results.
            $(results).addClass('jsonSuggest').hide();

            $(results).insertAfter(obj);
            jQuery(this).keyup(keyListener).
                keydown(function(e) {
                    // for tab key only
                    if (e.keyCode === 9 && currentSelection) {
                        $(currentSelection).trigger('click');
                        return true;
                    }
                }).
                blur(function(e) {
                    // We need to make sure we don't hide the result set
                    // if the input blur event is called because of clicking on
                    // a result item.
                    var resPos = $(results).offset();
                    resPos.bottom = resPos.top + $(results).height();
                    resPos.right = resPos.left + $(results).width();
                }).
                focus(function(e) {
                    if ($('li', results).length > 0) {
                        $(results).show();
                    }
                }).
                attr('autocomplete', 'off');

            // Escape the not character if present so that it doesn't act in the regular expression
            settings.notCharacter = regexEscape(settings.notCharacter || '');

            // Make sure the JSON data is a JavaScript object if given as a string.
            if (settings.data && typeof settings.data === 'string') {
                settings.data = $.parseJSON(settings.data);
            }
        });
    };

})(af);